@@ -10,26 +10,35 @@ import socket
 import collections
 import logging.config
 import logconfig
-import nbt
+from nbt.nbt import NBTFile
 import configurator
+import filecmp
+from magplus.magplus import MinecraftAssetsGetter
+import random
+import string
+import linux_metrics as lm
+
 
 # Config with logging config file
 logging.config.dictConfig(logconfig.LOGGING)
 logger = logging.getLogger('pyredstone')
 _version = '0.0.2'
 
-# Get server config
-#config = configurator.get_config()
-#session_name = 'troydoesntknow'
-#self.minecraft_dir = '/home/josh/minecraft'
-#bukkit = True
-#if bukkit:
-    #server_jar = 'craftbukkit.jar'
-#else:
-    #server_jar = 'minecraft_server.jar'
-#backup_dir = '/home/josh/minecraft_backup'
-#scp_server = 'josh@thepronserver'
-#scp_server_target = '/backup/minecraft'
+now = datetime.datetime.now()
+year = now.strftime("%y")
+week = now.strftime("%U")
+
+server_defaults = {"session_name": "troydoesntknow",
+"minecraft_dir": "/home/minecraft/minecraft",
+"server_jar": "vanilla.jar",
+"backup_dir": "/tmp",
+"mapper": "overviewer",
+"memory_min": "512",
+"memory_max": "1024",
+"java_args": "-XX:+AggressiveOpts",
+"debug": "False",
+"stable_releases": "False",
+}
 
 
 # Custom exceptions
@@ -53,64 +62,39 @@ class NotBukkitException(MinecraftException):
 
 
 class RedstoneServer:
-    def __init__(self, config_file=None, minecraft_dir=None, session_name=None, server_jar=None, backup_dir=None, mapper=None):
-        """ Create a new server wrapper.
-        If config_file is None, default config file will be created, first
-        in minecraft_dir (if not None), or current dir if None. Otherwise,
-        values are read in. If other args are not None, the config file will
-        be updated.
-        minecraft_dir: The default dir containing the server jar
-        session_name: The session name for Tmux or Screen
-        server_jar: The filename of the server jar (vanilla.jar or bukkit.jar)
-        backup_dir: The dir where backup tars and maps will be saved to
-        mapper: Name of mapping software. Choose from overviewer or mcmaps
+    def __init__(self, config_file):
+        """ Create a new server wrapper based on config_file
+        Config file must be specified. The required variables are
+        minecraft_dir and server_jar. The rest are optional and
+        will be set to defaults if not specified.
         """
-        # Try to find config file
-        write_config = False
-        if config_file is None:
-            if minecraft_dir is None or session_name is None or server_jar is None:
-                raise SyntaxError("You must specify either config_file, or all of minecraft_dir, session_name, and server_jar.")
-            if minecraft_dir is not None:
-                if os.path.exists(os.path.join(minecraft_dir, 'pyredstone.cfg')):
-                    config_file = os.path.join(minecraft_dir, 'pyredstone.cfg')
-                else:
-                    config_file = 'pyredstone.cfg'
-                    write_config = True
-        else:
-            logger.info('Asking configurator for config from %s' % config_file)
-            config = configurator.get_config(config_file)
-        # Set variables that might not get set to defaults
-        self.backup_dir = '/tmp'
-        self.mapper = 'overviewer'
-        # Load config file and
-        if os.path.exists(config_file):
-            #config = configurator.get_config(config_file)
-            self.minecraft_dir = config['minecraft_dir']
-            self.session_name = config['session_name']
-            self.server_jar = config['server_jar']
-            if 'backup_dir' in config:
-                self.backup_dir = config['backup_dir']
-            if 'mapper' in config:
-                self.mapper = config['mapper']
-        # Overwrite config is provided
-        if minecraft_dir is not None:
-            self.minecraft_dir = minecraft_dir
-        if session_name is not None:
-            self.session_name = session_name
-        if server_jar is not None:
-            self.server_jar = server_jar
-        if backup_dir is not None:
-            self.backup_dir = backup_dir
-        if mapper is not None:
-            self.mapper = mapper
-        # Write the config back to the config file
-        if write_config == True:
-            config['minecraft_dir'] = self.minecraft_dir
-            config['session_name'] = self.session_name
-            config['server_jar'] = self.server_jar
-            config['backup_dir'] = self.backup_dir
-            config['mapper'] = self.mapper
-            configurator.write_config(config_file, config)
+        if not os.path.exists(config_file):
+            raise SyntaxError("Could not find config file. Cannot continue.")
+        logger.info('Asking configurator for config from %s' % config_file)
+        config = configurator.get_config(config_file)
+
+        # Check for required bits of config.
+        if 'minecraft_dir' not in config or 'server_jar' not in config:
+            raise SyntaxError("Config file must specify minecraft_dir and server_jar")
+        # Check that minecraft_dir does exist. If not, try to make.
+        if not os.path.exists(config['minecraft_dir']):
+            try:
+                os.mkdir(config['minecraft_dir'])
+            except EnvironmentError as e:
+                logger.exception("Could not create minecraft_dir %s" % config['minecraft_dir'])
+                raise SyntaxError("Could not create minecraft_dir %s" % config['minecraft_dir'])
+        # Check that the server_jar is in the minecraft_dir. Otherwise, install! (Fail until install ready..)
+        if not os.path.exists(os.path.join(config['minecraft_dir'], config['server_jar'])):
+            logger.error("server_jar %s is not in minecraft_dir %s. Cannot continue." % (config['server_jar'], config['minecraft_dir']))
+            raise SyntaxError("server_jar %s is not in minecraft_dir %s. Cannot continue." % (config['server_jar'], config['minecraft_dir']))
+        # Set class attributes either from config or defaults if config doesn't have them.
+        for key, value in server_defaults.items():
+            if key not in config:
+                logger.debug("Setting %s to default: %s" % (key, value))
+                setattr(self, key, value)
+            else:
+                logger.debug("Setting %s: %s" % (key, config[key]))
+                setattr(self, key, config[key])
 
     ###
     # Convenience functions
@@ -156,24 +140,54 @@ class RedstoneServer:
         """
         line = line.replace('\x1b[0m', '')
         line = line.replace('\x1b[35m', '')
+        line = line.replace('\x1b[35;1m', '')
         line = line.replace('\x1b[m', '')
         line = line.replace('\n', '')
         return line
 
+    def download_file(self, url, output):
+        u = urllib2.urlopen(url)
+        print output
+        f = open(output, 'wb')
+        meta = u.info()
+        file_size = int(meta.getheaders("Content-Length")[0])
+        print "Downloading: %s Bytes: %s" % (output, file_size)
+
+        file_size_dl = 0
+        block_sz = 8192
+        while True:
+            buffer = u.read(block_sz)
+            if not buffer:
+                break
+
+            file_size_dl += len(buffer)
+            f.write(buffer)
+            status = r"%10d  [%3.2f%%]" % (file_size_dl, file_size_dl * 100. / file_size)
+            status = status + chr(8)*(len(status)+1)
+            print status,
+
+        f.close()
+
+    def test(self):
+        """ Here just for testing config and such. """
+        pass
+
     ###
     # Server commands
     ###
 
     def server_start(self):
-        """ Starts the Minecraft server.
-        Returns False if the server is already running or if the call to start the
-        server fails, True otherwise.
+        """ Starts the Minecraft server with args from config file.
+        Makes a warning log entry if already running and returns.
         """
 
         if self.status():
-            #print "Server already running in tmux session %s" % self.session_name
-            return False
-        cmd = 'tmux new -d -s %s "cd %s; java -Xms1524M -Xmx1524M -jar %s nogui"' % (self.session_name, self.minecraft_dir, self.server_jar)
+            logger.warning("Server already running.")
+            return
+        if self.java_args is None:
+            cmd = 'tmux new -d -s %s "cd %s; java -Xms%dM -Xmx%sM -jar %s nogui"' % (self.session_name, self.minecraft_dir, int(self.memory_min), int(self.memory_max), self.server_jar)
+        else:
+            cmd = 'tmux new -d -s %s "cd %s; java -Xms%dM -Xmx%sM %s -jar %s nogui"' % (self.session_name, self.minecraft_dir, int(self.memory_min), int(self.memory_max), self.java_args, self.server_jar)
         # Cannot use normal call command. Check output fails with tmux creation.
         try:
             subprocess.call(cmd, shell=True)
@@ -183,6 +197,7 @@ class RedstoneServer:
             #logging.exception("Command call error")
             raise MinecraftCommandException("Command '%s' failed with exit code: %d" % (cmd, e.returncode))
         #print "Minecraft started in tmux session %s" % self.session_name
+        logger.info("Server started successfully.")
         return True
 
     def server_stop(self, quick=False, msg=None):
@@ -208,6 +223,8 @@ class RedstoneServer:
             cmd = self.console_cmd("say Server going down in 15 seconds")
             time.sleep(15)
         self.console_cmd("stop")
+        logger.info("Server successfully stopped.")
+        return True
 
     def server_restart(self, quick=False, msg=None):
         """ Restarts the server, optionally giving warning messages.
@@ -219,8 +236,7 @@ class RedstoneServer:
             self.server_stop(quick, msg=msg)
             while self.status():
                 time.sleep(1)
-        self.server_start()
-        return self.status()
+        return self.server_start()
 
     def status(self):
         """ Checks whether the Minecraft server is running.
@@ -244,24 +260,166 @@ class RedstoneServer:
             return False
 
     def update(self):
-        """ Tries to update the server. Currently only supports vanilla updates.
-        Returns True if the server updated. Returns False otherwise.
+        """ Find latest version of server, compare to latest released version,
+        if there is a newer version, download and replace. If no version
+        number can be found, download the latest.
         """
-        #TODO Support CraftBukkit, CraftBukkit++ and other servers.
-        u = urllib2.urlopen('http://minecraft.net/download/minecraft_server.jar')
-        f = open('%s/test_update' % self.minecraft_dir, 'w')
-        f.write(u.read())
-        f.close()
-        testfile = file('%s/test_update' % self.minecraft_dir, 'rb')
-        currentfile = file('%s/minecraft_server.jar' % self.minecraft_dir, 'rb')
+        # Choose between vanilla and bukkit
+        download_list = []
+        mag = MinecraftAssetsGetter()
+        # Determine Jar type
+        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft_server.jar':
+            jar = 'vanilla'
 
-        if not zlib.adler32(testfile) == zlib.adler(currentfile):
-            server_stop()
-            shutil.move('%s/test_update' % self.minecraft_dir, '%s/minecraft_server' % self.minecraft_dir)
-            server_start()
-            return self.status()
-        return True
+        elif self.server_jar == 'craftbukkit.jar' or self.server_jar == 'bukkit.jar':
+            jar = 'bukkit'
+        else:
+            logger.error("server_jar is an unsupported version. Expect \
+            'vanilla.jar', 'minecraft_server.jar', ''craftbukkit.jar', or \
+            'bukkit.jar'. Got %s." % (self.server_jar))
+            raise SyntaxError("server_jar is an unsupported version. Expect \
+            'vanilla.jar', 'minecraft_server.jar', ''craftbukkit.jar', or \
+            'bukkit.jar'. Got %s." % (self.server_jar))
 
+        if self.stable_releases == "True":
+            stable = True
+            logger.info("Starting update for server type: %s - Stable" % (jar,))
+        elif self.stable_releases == "False":
+            stable = False
+            logger.info("Starting update for server type: %s - Unstable" % (jar,))
+        else:
+            logger.error("Stable releases must be either 'True' or 'False'. Got %s." % (self.stable_releases))
+            raise SyntaxError("Stable releases must be either 'True' or 'False'. Got %s." % (self.stable_releases))
+
+        # Find the existing version from the versions file in the minecraft_dir
+        versions_file = os.path.join(self.minecraft_dir, 'version.txt')
+        if os.path.exists(versions_file):
+            # Version file exists, we can proceed with comparison.
+            try:
+                with open(versions_file, 'r') as f:
+                    current_version = f.readline()
+            except IOError as e:
+                logger.exception("Could not read version file.")
+                raise IOError("Could not read version file %s" % versions_file)
+        else:
+            # No version file, just download new version.
+            current_version = None
+        logger.info("Current version is %s" % (current_version,))
+
+        # Get the link to the newest version.
+        if jar == 'vanilla':
+            ret = mag.getNewerVanillaVersion(current_version, stable)
+            if ret is None:
+                return
+            logger.debug(ret)
+            latest_version = mag.getVanillaServerUrl(ret['version'])
+        else:
+            ret = mag.getNewerBukkitVersion(current_version, stable)
+            if ret is None:
+                return
+            latest_version = ret['download_link']
+
+        logger.info("New version download link: %s" % (latest_version))
+        # Create a temp folder
+        random_string = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
+        try:
+            os.mkdir(os.path.join('/tmp', random_string))
+        except EnvironmentError as e:
+            logger.error("Cannot create temporary folder %s" % (os.path.join('/tmp', random_string)))
+            raise OSError("Cannot create temporary folder %s" % (os.path.join('/tmp', random_string)))
+        logger.info("Created temp directory %s" % (os.path.join('/tmp', random_string),))
+
+        # Download the file to the temp folder
+        self.download_file(latest_version, os.path.join('/tmp', random_string, self.server_jar))
+        if os.path.exists(os.path.join('/tmp', random_string, self.server_jar)):
+            logger.info("Downloaded file to %s" % os.path.join('/tmp', random_string, self.server_jar))
+        else:
+            logger.error("Download from %s to %s failed." % (latest_version, os.path.join('/tmp', random_string, self.server_jar)))
+            raise OSError("Download from %s to %s failed." % (latest_version, os.path.join('/tmp', random_string, self.server_jar)))
+
+        # Backup the existing server_jar to server_jar.bak
+        if os.path.exists(os.path.join(self.minecraft_dir, self.server_jar)):
+            try:
+                shutil.move(os.path.join(self.minecraft_dir, self.server_jar), os.path.join(self.minecraft_dir, self.server_jar + '.bak'))
+            except shutil.Error as e:
+                logger.error("Could not move old server_jar %s" % (os.path.join(self.minecraft_dir, self.server_jar)))
+                raise OSError("Could not move old server_jar %s" % (os.path.join(self.minecraft_dir, self.server_jar)))
+        logger.info("Backed up exist jar %s to %s." % (os.path.join(self.minecraft_dir, self.server_jar), os.path.join(self.minecraft_dir, self.server_jar)))
+
+        # Move the new server_jar from the temp folder to the minecraft_dir
+        try:
+            shutil.move(os.path.join('/tmp', random_string, self.server_jar), os.path.join(self.minecraft_dir, self.server_jar))
+        except shutil.Error as e:
+            logger.error("Could not move new server_jar %s to replace old server_jar %s" %(os.path.join()))
+            raise OSError("Could not move new server_jar %s to replace old server_jar %s" %(os.path.join()))
+        logger.info("Moved new jar to %s. Restart server to complete update." % (os.path.join(self.minecraft_dir, self.server_jar),))
+
+    def uptime(self):
+        ''' Returns the time the server host has been running since last reboot. '''
+        try:
+            with open('/proc/uptime', 'r') as f:
+                uptime_seconds = float(f.readline().split()[0])
+                uptime_string = str(datetime.timedelta(seconds = uptime_seconds))
+
+            return uptime_string
+        except EnvironmentError as e:
+            logger.exception("Could not get uptime.")
+            raise MinecraftException("Could not get uptime.")
+
+    def server_load(self):
+        ''' Average load on the server. '''
+        try:
+            return os.getloadavg()
+        except EnvironmentError as e:
+            logger.exception("Could not get load.")
+            raise MinecraftException("Could not get load.")
+
+    def server_memory(self):
+        ''' Returns memory in form: usedMB/totalMB. '''
+        # Get total free
+        mem_dict = {}
+        popen = os.popen("free -m").readlines()
+        mem_dict['total'] = popen[1].split()[1]
+        mem_dict['free'] = popen[1].split()[3]
+        mem_dict['used'] = popen[1].split()[2]
+        mem_dict['used_minus_cache'] = popen[2].split()[2]
+        mem_dict['free_minus_cache'] = popen[2].split()[3]
+        return mem_dict
+
+    def server_stats(self, disk='sda', net='eth0', sample_duration=0.1):
+        """ Returns a dict of stats about the server, including CPU, memory,
+        disk and networking activity. Disk is a disk device in /dev, net is
+        the name of a networking device. Sample duration is how long the sampling
+        will take. 0.1 is low but will provide a better user experience through
+        a web interface.
+
+        {procs_running: number of processes currently running
+        cpu_use: CPU usage as a percentage.
+        cpu_iowait: Percent of time waiting for I/O in CPU time.
+        disk_busy: Amount of time the disk is busy
+        disk_reads: Total disk reads since last reboot
+        disk_writes: Total disk writes since last reboot.
+        }
+
+        """
+        stats_dict = {}
+        stats_dict['procs_running'] = lm.cpu_stat.procs_running()
+        cpu_stats = lm.cpu_stat.cpu_percents(sample_duration)
+        stats_dict['cpu_use'] = 100 - cpu_stats['idle']
+        stats_dict['cpu_iowait'] = cpu_stats['iowait']
+        stats_dict['disk_busy'] = lm.disk_stat.disk_busy(disk, sample_duration)
+        #r, w = lm.disk_stat.disk_reads_writes(disk)
+        #stats_dict['disk_reads'] = r
+        #stats_dict['disk_writes'] = w
+        used, total = lm.mem_stat.mem_stats()
+        # In Megabytes
+        stats_dict['mem_used'] = used / 1048576
+        stats_dict['mem_total'] = total / 1048576
+        # In Kilobits
+        rx_bits, tx_bits = lm.net_stat.rx_tx_bits(net)
+        stats_dict['net_tx'] = tx_bits / 1000
+        stats_dict['net_rx'] = rx_bits / 1000
+        print stats_dict
     ###
     # Server Settings
     ###
@@ -405,6 +563,38 @@ class RedstoneServer:
             logger.exception()
         return True
 
+    def rotate_logs(self, quick=False):
+        """ Rotates logs from server.log to logs/server.log-YYYY-MM-DD.counter
+        """
+        # Check that log file exists
+        log_file = os.path.join(self.minecraft_dir, 'server.log')
+        if not os.path.exists(log_file):
+            raise MinecraftException("No log file at %s" % log_file)
+        logs_dir = os.path.join(self.minecraft_dir, 'logs')
+        # Create logs_dir if it does not exist.
+        if not os.path.exists(logs_dir):
+            try:
+                os.mkdir(logs_dir)
+            except EnvironmentError as e:
+                raise MinecraftException("Logs directory %s did not exists and could not be created." % logs_dir)
+        # Can't rotate logs if server is running.
+        if self.status():
+            self.server_stop(quick=quick)
+        # Move server.logs into logs/ with name based on date. First check to see if same name exists.
+        old_log_file = os.path.join(logs_dir, 'server.log' + '-' + datetime.date.today().isoformat())
+        counter = 0
+        # If old_log_file exists, find a new logfile name, in form of logs/server.log-YYYY-MM-DD.counter
+        while os.path.exists(old_log_file):
+            old_log_file = os.path.join(logs_dir, 'server.log' + '-' + datetime.date.today().isoformat() + '.' + counter)
+            counter = counter + 1
+        # Move the log file
+        try:
+            shutil.move(log_file, old_log_file)
+        except shutil.Error as e:
+            raise MinecraftException("Could not rotate log from %s to %s" % (log_file, old_log_file))
+        # 
+        self.server_start()
+
     ###
     # In game status (NBT)
     ###
@@ -413,7 +603,7 @@ class RedstoneServer:
         """ Finds the spawn coordinates. Returns a 3tuple of ints in the format
         (X, Y, Z) or None if the coordinates cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -423,7 +613,7 @@ class RedstoneServer:
         """ Finds the seed of the server. Returns the seed as a string, or
         None if the seed cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -434,7 +624,7 @@ class RedstoneServer:
         thundering, False if not thundering, and None if the thundering state
         cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -448,7 +638,7 @@ class RedstoneServer:
         raining, False if not raining, and None if the raining state
         cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -461,7 +651,7 @@ class RedstoneServer:
         """ Gets the current in game time. Returns the time as an int between
         0 and 23999, or None if the time cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -471,7 +661,7 @@ class RedstoneServer:
         """ Gets the current number of elapsed in game days. Returns the days
         as an int, or None if the days cannot be found.
         """
-        n = nbt.NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
+        n = NBTFile('%s/%s/level.dat' % (self.minecraft_dir, self.session_name))
         if n == None:
             return None
         else:
@@ -516,7 +706,30 @@ class RedstoneServer:
         self.console_cmd("list")
         count = 0
         ret_list = []
-        for line in reversed(open("%s/server.log" % self.minecraft_dir).readlines()):
+        with open("%s/server.log" % self.minecraft_dir, "r") as f:
+            f.seek (0, 2)           # Seek @ EOF
+            fsize = f.tell()        # Get Size
+            f.seek(max(fsize-1024, 0), 0) # Set pos @ last n chars
+            lines = f.readlines()       # Read to end
+
+        lines = list(reversed(lines[-20:]))    # Get last 10 lines
+        # This returns True if any line is exactly find_str + "\n"
+        for line in lines:
+            if "players online:" in line and count < 20:
+                #print "Winning line: #", count, " ", line
+                # Find the number of players.
+                for chars in line.split():
+                    if '/' in chars:
+                        connected, max_players = chars.split('/')
+                        #print connected, max_players
+                        #print lines[count - int(connected):count]
+                        players = lines[count - int(connected):count]
+                        for player in players:
+                            stripped_player = player.replace('\x1b[m', '').strip()
+                            ret_list.append(stripped_player)
+                            #print "Stripped ", stripped_player
+                        return ret_list
+            # Code for 1.2.5 and previous..
             if "Connected players" in line and count < 20:
                 #print "winning line: ", line[:-5]
                 players = line[:-4].split()[5:]
@@ -524,7 +737,7 @@ class RedstoneServer:
                 break
             else:
                 count += 1
-                print line
+                #print "Odd line ", line
                 continue
             # Remove commas from players
             for player in players:
@@ -532,6 +745,73 @@ class RedstoneServer:
             break
         return ret_list
 
+    def get_player_skin(self, player):
+        """ Get the URL to a player's skin, or None if does not exist.
+        """
+        conn = httplib.HTTPConnection('http://s3.amazonaws.com/MinecraftSkins/%s.png' % player)
+        conn.request('HEAD', path)
+        response = conn.getresponse()
+        conn.close()
+        if response.status == 200:
+            return 'http://s3.amazonaws.com/MinecraftSkins/%s.png' % player
+        else:
+            return None
+
+    def get_player_inventory(self, player):
+        n = self._get_player_nbt(player)
+        #n = NBTFile('/home/josh/Downloads/servercobra.dat')
+
+        inv_list = []
+        inv = n["Inventory"]
+        for item in inv:
+            #print(item.pretty_tree())
+            inv_dict = {}
+            inv_dict['id'] = item['id'].value
+            if 'Count' in item:
+                inv_dict['count'] = item['Count'].value
+            else:
+                logger.warning("No count?")
+            if 'Slot' in item:
+                inv_dict['slot'] = item['Slot'].value
+            else:
+                logger.warning("No slot?")
+            if 'tag' in item:
+                inv_dict['enchantments'] = {}
+                if 'ench' in item['tag']:
+                    for ench in item['tag']['ench']:
+                        inv_dict['enchantments']['id'] = ench['id'].value
+                        inv_dict['enchantments']['level'] = ench['lvl'].value
+            inv_list.append(inv_dict)
+        return inv_list
+
+    def get_player_location(self, player):
+        n = self._get_player_nbt(player)
+        loc_list = n["Pos"]
+        return (loc_list[0].value, loc_list[1].value, loc_list[2].value)
+
+    def get_player_spawn(self, player):
+        n = self._get_player_nbt(player)
+        return (n["SpawnX"].value, n["SpawnY"].value, n["SpawnZ"].value)
+
+    def get_player_health(self, player):
+        n = self._get_player_nbt(player)
+        return n["Health"].value
+
+    def get_player_xp(self, player):
+        n = self._get_player_nbt(player)
+        return n["XpLevel"].value
+
+    def _get_player_nbt(self, player):
+        self.console_cmd("save-all")
+        nbt_file = os.path.join(self.minecraft_dir, self.session_name, 'players', player + '.dat')
+        if not os.path.exists(nbt_file):
+            raise MinecraftException("Could not find player file for player: %s" % player)
+        return NBTFile(nbt_file)
+
+    def delete_player_item(self, player, slot):
+        """ Will delete everything in the given slot from the player's inventory. """
+        pass
+
     def get_logs(self, num_lines=-1, log_filter=None, chat_filter=None):
         """ Get a number of lines from the log in reverse order (-1 for all).
         log_filter can be chat, players, or None.
@@ -550,15 +830,15 @@ class RedstoneServer:
         count = 0
         ret_list = []
         for line in reversed(open(logfile).readlines()):
-            if chat_filter == 'chat' and "<" in line and ">" in line and "[INFO]" in line:
+            if log_filter == 'chat' and "<" in line and ">" in line and "[INFO]" in line:
                 l = self._sanitize_log_line(line).split()
                 ret_list.append(("chat", l[0], l[1], l[3], " ".join(l[4:])))
                 count += 1
-            elif chat_filter == 'chat' and "[Server]" in line:
+            elif log_filter == 'chat' and "[Server]" in line and "World backed up!" not in line and "Server going down in" not in line:
                 l = self._sanitize_log_line(line).split()
                 ret_list.append(("chat", l[0], l[1], l[3], " ".join(l[4:])))
                 count += 1
-            elif chat_filter == 'players':
+            elif log_filter == 'players':
                 if "logged in" in line or "logged out" in line or "lost connection" in line:
                     l = self._sanitize_log_line(line).split()
                     if "logged in" in line:
@@ -567,7 +847,7 @@ class RedstoneServer:
                         action = "logged out"
                     ret_list.append(("players", l[0], l[1], l[3], action))
                     count += 1
-            elif chat_filter == None:
+            elif log_filter == None:
                 ret_list.append(("none", self._sanitize_log_line(line)))
                 count += 1
             if num_lines > 0:
@@ -691,6 +971,7 @@ class RedstoneServer:
             if self.is_banned(player_or_ip, 'player'):
                 return False
             self.console_cmd("ban %s" % (player_or_ip))
+        return True
 
     def pardon(self, player_or_ip):
         """ Removes the ban on a player or IP. Attempts to determine if the arg is
@@ -740,7 +1021,7 @@ class RedstoneServer:
                     else:
                         ops.append(user)
             return ops
-        logging.error("Could not find an ops file at %s/ops.txt" % self.minecraft_dir)
+        logger.error("Could not find an ops file at %s/ops.txt" % self.minecraft_dir)
         raise MinecraftException("Could not find an ops file at %s/ops.txt" % self.minecraft_dir)
 
     def get_whitelist(self):
@@ -757,34 +1038,43 @@ class RedstoneServer:
     def disable_whitelist(self):
         """ Disables the whitelist. All users can log in. """
         self.console_cmd("whitelist off")
+        return True
 
     def enable_whitelist(self):
         """ Prevents users not on the whitelist from connecting. Ops may
         always connect. """
         self.console_cmd("whitelist on")
+        return True
 
     # Renew whitelist from disk. Call after adding or removing from whitelist.
     def _whitelist_reload(self):
         """ Reloads the whitelist so changes take affect.
         """
         self.console_cmd("whitelist reload")
+        return True
 
     def add_to_whitelist(self, player):
-        """ Adds a player to the whitelist. Fails silently if player is already
+        """ Adds a player to the whitelist. Returns False if player is already
         on the whitelist. Raises MinecraftCommandException if server command
         fails.
         """
         if player not in self.get_whitelist():
             self.console_cmd("whitelist add %s" % (player))
             self._whitelist_reload()
+            return True
+        else:
+            return False
 
     def remove_from_whitelist(self, player):
         """ Removes a player from the whitelist. Fails silently if player is
         not on the whitelist. Raises MinecraftCommandException if server command
         fails.
         """
+        if player not in self.get_whitelist():
+            return False
         self.console_cmd("whitelist remove %s" % (player))
         self._whitelist_reload()
+        return True
 
     ###
     #Plugins
@@ -847,7 +1137,7 @@ class RedstoneServer:
         raises NotBukkitException if server is not running Bukkit.
         """
 
-        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft.jar':
+        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft_server.jar':
             raise NotBukkitException("No plugins on vanilla Minecraft server.")
         return name in self.list_plugins()
 
@@ -858,7 +1148,7 @@ class RedstoneServer:
 
         raises NotBukkitException if server is not running Bukkit.
         """
-        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft.jar':
+        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft_server.jar':
             raise NotBukkitException("No plugins on vanilla Minecraft server.")
         # Check if plugin is enabled
         if not os.path.exists("%s/plugins/%s.jar" % (self.minecraft_dir, name)):
@@ -883,7 +1173,7 @@ class RedstoneServer:
         raises NotBukkitException if server is not running Bukkit.
         """
 
-        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft.jar':
+        if self.server_jar == 'vanilla.jar' or self.server_jar == 'minecraft_server.jar':
             raise NotBukkitException("No plugins on vanilla Minecraft server.")
         # Check if plugin is already enabled
         if name + '.jar' in os.listdir("%s/plugins" % self.minecraft_dir):
@@ -1019,7 +1309,7 @@ class RedstoneServer:
         player is not connected.
         """
         if player not in self.get_players():
-            logging.info("Tried to whisper to player %s, who is not currently connected." % (player))
+            logger.info("Tried to whisper to player %s, who is not currently connected." % (player))
         self.console_cmd("tell %s %s" % (player, message))
 
 if __name__ == '__main__':
